'ImgVerify - Image Verification Class for .NET
'Copyright (C) 2007  Bobby Nakanelua (bnakan@gmail.com)

'This library is free software; you can redistribute it and/or
'modify it under the terms of the GNU Lesser General Public
'License as published by the Free Software Foundation; either
'version 2.1 of the License, or (at your option) any later version.

'This library is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'Lesser General Public License for more details.

'You should have received a copy of the GNU Lesser General Public
'License along with this library; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
'
'

Imports Microsoft.VisualBasic
Imports System.Drawing
Imports System.Security.Cryptography

Namespace Nakan

    Public Class ImgVerify

#Region "PRIVATE DECLARATIONS"

        'Private Properties
        Private _gstrMasterSeed As String = "a@4534fbXas4gb@%&**bxc"
        Private _gintCodeLen As Integer = 5
        Private _gstrFolder As String = ""
        Private _gstrHash As String = ""
        Private _gintCase As Integer = 0

        Private _gintBkgColor As Integer = 7
        Private _gintLetterColor As Integer = 0
        Private _gintLineColor As Integer = 6

#End Region

#Region "PRIVATE PROCEDURES"

        Private Function MD5Text(ByVal strText As String) As String
            'This function will MD5 the passed text and return the hex value
            Try
                Dim i As Integer = 0
                Dim strTemp As String = ""
                Dim strH As String = ""
                Dim bytHashedData As Byte()
                Dim uteEncoder As New UTF8Encoding()
                Dim md5SrvProv As New MD5CryptoServiceProvider()
                bytHashedData = md5SrvProv.ComputeHash(uteEncoder.GetBytes(strText))

                For i = 0 To (bytHashedData.Length - 1)
                    strH = Hex(bytHashedData(i))
                    If strH.Length = 1 Then
                        strH = "0" & strH
                    End If
                    strTemp &= strH
                Next
                Return strTemp
            Catch ex As Exception
                Return ""
            End Try
        End Function

        Private Function ReturnBrush(Optional ByVal intIndex As Integer = 0) As Brush
            'This function will return a brush based on the passed index
            Try
                Select Case intIndex
                    Case 0
                        Return Brushes.Black
                    Case 1
                        Return Brushes.Blue
                    Case 2
                        Return Brushes.Purple
                    Case 3
                        Return Brushes.Red
                    Case 4
                        Return Brushes.Yellow
                    Case 5
                        Return Brushes.Green
                    Case 6
                        Return Brushes.LightGray
                    Case 7
                        Return Brushes.White
                    Case Else
                        Return Brushes.Black
                End Select
            Catch ex As Exception
                Return Brushes.Black
            End Try
        End Function

        Private Function ReturnPen(Optional ByVal intIndex As Integer = 6) As Pen
            'This function will return a pen based on the passed index
            Try
                Select Case intIndex
                    Case 0
                        Return Pens.Black
                    Case 1
                        Return Pens.Blue
                    Case 2
                        Return Pens.Purple
                    Case 3
                        Return Pens.Red
                    Case 4
                        Return Pens.Yellow
                    Case 5
                        Return Pens.Green
                    Case 6
                        Return Pens.LightGray
                    Case 7
                        Return Pens.White
                    Case Else
                        Return Pens.LightGray
                End Select
            Catch ex As Exception
                Return Pens.Black
            End Try
        End Function

        Private Function ReturnColor(Optional ByVal intIndex As Integer = 0) As Color
            'This function will return a color based on the passed index
            Try
                Select Case intIndex
                    Case 0
                        Return Color.Black
                    Case 1
                        Return Color.Blue
                    Case 2
                        Return Color.Purple
                    Case 3
                        Return Color.Red
                    Case 4
                        Return Color.Yellow
                    Case 5
                        Return Color.Green
                    Case 6
                        Return Color.LightGray
                    Case 7
                        Return Color.White
                    Case Else
                        Return Color.Black
                End Select
            Catch ex As Exception
                Return Color.Black
            End Try
        End Function

        Private Function ReturnRndString(ByVal intLength As Integer, Optional ByVal intCase As Integer = 0) As String
            'This function will return a random number based on the passed length.
            'CASE VALUES:
            '   0   = Mixed Case (DEFAULT)
            '   1   = Lower Case
            '   2   = Upper Case
            '
            '97 / 122   'LOWER
            '65 / 90    'UPPER
            '48 / 57    ' 0-9
            Try
                Dim objRnd As New Random
                Dim intR As Integer = 0
                Dim l As Integer = 0
                Dim strTemp As String = ""

                For l = 1 To intLength
                    intR = objRnd.Next(1, 4)
                    Select Case intR
                        Case 1
                            'Number
                            strTemp &= Chr(objRnd.Next(48, 57))
                        Case 2
                            'Upper
                            strTemp &= Chr(objRnd.Next(65, 90))
                        Case Else
                            'Lower Case
                            strTemp &= Chr(objRnd.Next(97, 122))
                    End Select
                Next

                Select Case intCase
                    Case 0
                        'MIXED
                        Return strTemp
                    Case 1
                        'LOWER
                        Return strTemp.ToLower
                    Case Else
                        'UPPER
                        Return strTemp.ToUpper
                End Select
            Catch ex As Exception
                Return ""
            End Try
        End Function

#End Region

#Region "PUBLIC PROPERTIES"

        Public Property CodeLength() As Integer
            Get
                CodeLength = _gintCodeLen
            End Get
            Set(ByVal value As Integer)
                If (value < 1) Or (value > 8) Then
                    _gintCodeLen = 5
                Else
                    _gintCodeLen = value
                End If
            End Set
        End Property

        Public Property ImageFolder() As String
            Get
                ImageFolder = _gstrFolder
            End Get
            Set(ByVal value As String)
                Try
                    value = value.Replace("\", "/")

                    If Right(value, 1) <> "/" Then
                        _gstrFolder = value & "/"
                    Else
                        _gstrFolder = value
                    End If
                Catch ex As Exception
                    _gstrFolder = ""
                End Try
            End Set
        End Property

        Public ReadOnly Property ImageHash() As String
            Get
                ImageHash = _gstrHash
            End Get
        End Property

        Public Property BackgroundColor() As Integer
            Get
                BackgroundColor = _gintBkgColor
            End Get
            Set(ByVal value As Integer)
                If (value >= 0) And (value <= 7) Then
                    _gintBkgColor = value
                End If
            End Set
        End Property

        Public Property LetterColor() As Integer
            Get
                LetterColor = _gintLetterColor
            End Get
            Set(ByVal value As Integer)
                If (value >= 0) And (value <= 7) Then
                    _gintLetterColor = value
                End If
            End Set
        End Property

        Public Property LineColor() As Integer
            Get
                LineColor = _gintLineColor
            End Get
            Set(ByVal value As Integer)
                If (value >= 0) And (value <= 7) Then
                    _gintLineColor = value
                End If
            End Set
        End Property

        Public WriteOnly Property MasterSeed() As String
            Set(ByVal value As String)
                _gstrMasterSeed = value
            End Set
        End Property

        Public Property LetterCase() As Integer
            Get
                LetterCase = _gintCase
            End Get
            Set(ByVal value As Integer)
                If (value >= 0) And (value <= 2) Then
                    _gintCase = value
                Else
                    _gintCase = 0
                End If
            End Set
        End Property

#End Region

#Region "PUBLIC PROCEDURES"

        Public Function ValidCode(ByVal strCode As String, ByVal strSeed As String, ByVal strHash As String) As Boolean
            'This function will check to see whether the passed hash and code is valid 
            Try
                Dim strH As String = ""
                strH = MD5Text(strCode & "!" & _gstrMasterSeed & "!" & strSeed)
                If strH.ToLower = strHash.ToLower Then
                    ValidCode = True
                Else
                    ValidCode = False
                End If
            Catch ex As Exception
                ValidCode = False
            End Try
        End Function

        Public Function GenerateImage(ByVal strSeed As String) As String
            'This function will generate the random image and return the path to the image
            Try
                Dim strText As String = ReturnRndString(_gintCodeLen, _gintCase)
                Dim l As Integer = 0
                Dim s As Integer = 0
                Dim intStart As Integer = 0
                Dim intMaxShapes As Integer = 5
                Dim intLetterWidth As Integer = 22
                Dim objRnd As New Random
                Dim strHash As String = ""

                Dim bmpImage As Bitmap = New Bitmap(200, 50, Imaging.PixelFormat.Format24bppRgb)
                Dim objGraphics As Graphics = Graphics.FromImage(bmpImage)

                'Reset the Hash Property
                _gstrHash = ""

                'This will center the text in the image
                intStart = (200 - ((strText.Length) * intLetterWidth)) / 2

                'Set the background color
                objGraphics.Clear(ReturnColor(_gintBkgColor))

                'Draw the lines
                For s = 0 To intMaxShapes
                    If s Mod 2 = 1 Then
                        objGraphics.DrawLine(ReturnPen(_gintLineColor), objRnd.Next(1, 100), 1, objRnd.Next(100, 200), 50)
                    Else
                        objGraphics.DrawLine(ReturnPen(_gintLineColor), objRnd.Next(100, 200), 1, objRnd.Next(1, 100), 50)
                    End If
                Next

                'Draw the letters
                For l = 0 To (strText.Length - 1)
                    objGraphics.DrawString(strText.Substring(l, 1), New Font("Times New Roman", objRnd.Next(17, 21), FontStyle.Italic), ReturnBrush(_gintLetterColor), (intLetterWidth * l) + intStart, objRnd.Next(1, 15))
                Next

                'Get the Hash
                strHash = MD5Text(strText & "!" & _gstrMasterSeed & "!" & strSeed)
                'Set the Hash Property
                _gstrHash = strHash

                'Save the Image
                bmpImage.Save(System.Web.HttpContext.Current.Server.MapPath(".") & "\" & _gstrFolder & strHash & ".jpg", Imaging.ImageFormat.Jpeg)

                'Perform Cleanup
                objGraphics.Dispose()
                bmpImage.Dispose()

                Return _gstrFolder & strHash & ".jpg"
            Catch ex As Exception
                Return ""
            End Try
        End Function

#End Region

    End Class

End Namespace